<?php

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;
use App\Repository\CountryRepository;
use App\Entity\Abstraction\BaseEntity;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * @ORM\Entity(repositoryClass=CountryRepository::class)
 */
class Country extends BaseEntity
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=4)
     */
    private $code;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $name;

    /**
     * @ORM\OneToMany(targetEntity=CountryRegion::class, mappedBy="country")
     */
    private $regions;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $subname;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $localeName;

    public function __construct()
    {
        $this->regions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    /**
     * @return Collection|CountryRegion[]
     */
    public function getRegions(): Collection
    {
        return $this->regions ?? new ArrayCollection();
    }

    public function addRegion(CountryRegion $region): self
    {
        if (!$this->regions->contains($region)) {
            $this->regions[] = $region;
            $region->setCountry($this);
        }

        return $this;
    }

    public function removeRegion(CountryRegion $region): self
    {
        if ($this->regions->removeElement($region)) {
            // set the owning side to null (unless already changed)
            if ($region->getCountry() === $this) {
                $region->setCountry(null);
            }
        }

        return $this;
    }

    public function getSubname(): ?string
    {
        return $this->subname;
    }

    public function setSubname(?string $subname): self
    {
        $this->subname = $subname;

        return $this;
    }

    public function getLocaleName(): ?string
    {
        return $this->localeName;
    }

    public function setLocaleName(?string $localeName): self
    {
        $this->localeName = $localeName;

        return $this;
    }
}
