import { HttpClient } from '@angular/common/http';
import { EventEmitter, Injectable, ModuleWithProviders } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { LoginModel } from '../../models/login.model';
import { RegisterModel } from '../../models/register.model';
import { TokenResponse } from '../../models/token.response';
import { UserModel } from '../../models/user.model';
import { AuthTokenService } from '../../../shared/services/auth/auth-token.service';
import { Router } from '@angular/router';

@Injectable({
    providedIn: 'root',
})
export class AuthService {

    private static updateAgent: any;
    private user: UserModel | null = null;
    public userChange = new EventEmitter<UserModel | null>();

    constructor(
        private http: HttpClient,
        private authTokenService: AuthTokenService,
        private router: Router,
    ) {
        this.userChange.subscribe(user => {
            this.user = user;
        });
    }

    getUser(): UserModel | null {
        return this.user;
    }

    checkUser(): Promise<UserModel> {
        return new Promise<UserModel>((resolve, reject) => {
            this.http.post<UserModel>('/api/user-check', {}).subscribe(user => {
                this.userChange.emit(user);
                if (AuthService.updateAgent) {
                    clearTimeout(AuthService.updateAgent);
                }
                AuthService.updateAgent = setTimeout(() => {
                    this.checkUser();
                }, 2000) as any;
                resolve(user);
            }, err => {
                if (err.status === 401) {
                    this.logout();
                }
                reject(err);
            });
        });
    }

    createAccount(data: RegisterModel): Observable<UserModel> {
        return this.http.post<UserModel>('/api/register', data);
    }

    login(data: LoginModel): Observable<TokenResponse> {
        const out = new Subject<TokenResponse>();
        this.http.post<TokenResponse>('/api/login', {
            username: data.email,
            password: data.password,
        }).subscribe(response => {
            this.authTokenService.setToken(response.token);
            this.checkUser().then(() => {
                out.next(response);
            }, () => {
                out.next(null);
            }).catch(() => {
                out.next(null);
            });
        });
        return out;
    }

    logout(): void {
        if (AuthService.updateAgent) {
            clearTimeout(AuthService.updateAgent);
        }
        this.authTokenService.removeToken();
        this.userChange.emit(null);
        this.router.navigate(['/auth']);
    }
}
