import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@ng-stack/forms';
import { Observable, Subject } from 'rxjs';
import { CountryModel } from 'src/app/modules/shared/models/country.model';
import { CountryService } from 'src/app/modules/shared/services/country/country.service';
import { map, startWith } from 'rxjs/operators';
import { AuthService } from 'src/app/modules/auth/services/auth/auth.service';
import { ProfileEditComponent } from '../profile-edit/profile-edit.component';

interface PhoneEntry{
    number: string;
    name: string;
    description: string;
    icon: string;
}
interface EmailEntry{
    number: string;
    name: string;
    description: string;
    icon: string;
}

class ProfileInformationBasicModel {
    name: string;
    surname: string;

    country: number;
    state: string;
    city: string;
    zip: string;
    phones: PhoneEntry[];
    emails: EmailEntry[];

}
@Component({
    selector: 'app-profile-edit-basics',
    templateUrl: './profile-edit-basics.component.html',
    styleUrls: ['./profile-edit-basics.component.scss']
})
export class ProfileEditBasicsComponent implements OnInit {

    form: FormGroup<ProfileInformationBasicModel>;
    filteredCountries: Observable<CountryModel[]>;
    countryInputChnage = new Subject<string>();
    countries: CountryModel[] = [];
    countryInput = '';
    @ViewChild('footer') footerElement: ElementRef;

    constructor(
        private countryService: CountryService,
        authService: AuthService,
        formBuilder: FormBuilder,
        parent: ProfileEditComponent,
    ) {
        const phones = formBuilder.array<PhoneEntry>([]);
        const emails = formBuilder.array<EmailEntry>([]);
        const user = authService.getUser();

        this.form = formBuilder.group<ProfileInformationBasicModel>({
            name: parent.person.name,
            surname: parent.person.surname,

            country: parent.person.country,
            state: 'state',
            city: 'city',
            zip: 'zip',
            phones,
            emails,
        });

        this.countryService.getCountries().subscribe(countries => {
            this.countries = countries;
            this.countryInputChnage.next(this.countryInput);
            this.form.patchValue({
                country: this.form.value.country,
            });
        });
        this.filteredCountries = this.countryInputChnage.pipe(
          startWith(''),
          map(name => name ? this.filterCountries(name) : this.countries.slice())
        );
    }

    displayCountry = (countryId: number) => {
        const country = this.countries.find(i => i.id === countryId);
        return country ? country.name : '';
    }

    ngOnInit(): void {
    }

    filterCountries(name: string): CountryModel[] {
        return this.countries.filter(i => i.name.toLowerCase().includes(name));
    }

    countryFieldChanged(event): void {
        this.countryInput = event.target.value.toLowerCase();
        this.countryInputChnage.next(this.countryInput);
    }
}
