import { Component, OnInit } from '@angular/core';
import { FormControl } from '@angular/forms';
import { AppService } from 'src/app/modules/shared/services/app/app.service';
import { ThemeDefinition, ThemeService } from 'src/app/modules/shared/services/theme/theme.service';

@Component({
    selector: 'app-profile-edit-settings',
    templateUrl: './profile-edit-settings.component.html',
    styleUrls: ['./profile-edit-settings.component.scss']
})
export class ProfileEditSettingsComponent implements OnInit {
    themeControl = new FormControl();
    langControl = new FormControl();
    lang: string;
    theme: string;
    langs: string[] = [];
    themes: ThemeDefinition[] = [];

    constructor(
        private themeService: ThemeService,
        private appService: AppService,
    ) {
        this.configureThemeEvents();
        this.configureLanguageEvents();
    }

    ngOnInit(): void {
    }

    configureLanguageEvents(): void {
        this.langs = this.appService.getLangs();
        this.lang = this.appService.getLang();
        this.appService.changeLang(this.lang);
        this.langControl.valueChanges.subscribe(lang => {
            this.appService.changeLang(lang);
        });
        this.onLangChanged(this.appService.getLang());
        this.appService.langChange.subscribe(lang => {
            this.onLangChanged(lang);
        });
    }

    configureThemeEvents(): void {
        this.themes = this.themeService.getThemes();
        this.themeControl.valueChanges.subscribe(theme => {
            this.themeService.setTheme(theme);
        });
        this.onThemeChanged(this.themeService.getTheme());
        this.themeService.themeChange.subscribe(theme => {
            this.onThemeChanged(theme);
        });
    }

    onThemeChanged(theme: string): void {
        if (theme === undefined) {
            return;
        }
        this.theme = theme;
        this.themeControl.setValue(theme);
    }

    onLangChanged(lang: string): void {
        if (lang === undefined) {
            return;
        }
        this.lang = lang;
        this.langControl.setValue(lang);
    }

}
