import { HttpInterceptor, HttpRequest, HttpHandler, HttpEvent } from '@angular/common/http';
import { Injectable, Injector } from '@angular/core';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { NotificationService } from '../../notification/notification.service';

@Injectable()
export class ErrorHandlerInterceptor implements HttpInterceptor {

    get notificationService(): NotificationService {
        return this.injector.get(NotificationService);
    }

    constructor(
        private injector: Injector,
    ) {
    }

    intercept(httpRequest: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        return next.handle(httpRequest).pipe(
          catchError(response => {
                switch (response.status) {
                    case 406:
                        if (response.error instanceof Blob && response.error.type === 'application/json') {
                            const result = new Promise((resolve) => {
                                const reader = new FileReader();
                                reader.onload = (e: Event) => {
                                    resolve(JSON.parse((e.target as any).result));
                                };
                                reader.readAsText(response.error);
                            });
                            result.then((data) => {
                                response.error = data;
                                this.displayNotificationIfValidationError(response);
                            });
                            break;
                        } else {
                            this.displayNotificationIfValidationError(response);
                            break;
                        }
                }
                return throwError(response);
            })
        );
    }

    displayNotificationIfValidationError(response): void {
        if (response?.error?.data) {
            this.notificationService.error(response.error.data);
        }
    }
}
